"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createIpcServer = createIpcServer;
exports.createIpcClient = createIpcClient;

var _os = _interopRequireDefault(require("os"));

var _crypto = _interopRequireDefault(require("crypto"));

var _createIpcRequestHandler = require("./create-ipc-request-handler");

var _nodeIpc = require("node-ipc");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// @ts-expect-error these types need updating
function randomId() {
  return _crypto.default.randomBytes(8).toString('hex');
}

function getPath(ipc, id) {
  return `${ipc.config.socketRoot}${ipc.config.appspace}${id}`;
}

/**
 * Creates an inter-process communication server between the main build process
 * and separate processes created by next.js for generating pages in parallel.
 *
 * This uses the package [`node-ipc`][gh] and uses Unix sockets/Windows
 * sockets to communicate. A socket ID (to be passed to `createIpcClient`) is
 * returned
 *
 * [gh]: https://github.com/RIAEvangelist/node-ipc
 */
async function createIpcServer({
  nodeIpcConfig = {
    silent: true
  }
} = {}) {
  const ipc = new _nodeIpc.IPCModule(); // const ipc = require('node-ipc').default as typeof import('node-ipc');

  const ipcRequestHandler = (0, _createIpcRequestHandler.createIpcRequestHandler)();
  const id = `next-plugin-preval-${randomId()}.${_os.default.hostname()}`;
  Object.assign(ipc.config, nodeIpcConfig, {
    id
  });
  const promise = new Promise((resolve, reject) => {
    ipc.serve(getPath(ipc, id), () => resolve());
    ipc.server.on('error', reject);
  });
  ipc.server.on('request', async (message, socket) => {
    const result = await ipcRequestHandler(message.body.resource);
    const responseMessage = {
      id: message.id,
      body: {
        result
      }
    };
    ipc.server.emit(socket, 'response', responseMessage);
  });
  ipc.server.start();
  await promise;
  return id;
}
/**
 * Creates an inter-process communication client between the main build process
 * and separate processes created by next.js for generating pages in parallel.
 *
 * This uses the package [`node-ipc`][gh] and uses Unix sockets/Windows
 * sockets to communicate. Takes in a socket ID and returns an object with a
 * request function.
 *
 * [gh]: https://github.com/RIAEvangelist/node-ipc
 */


async function createIpcClient(id, {
  nodeIpcConfig = {
    silent: true
  }
} = {}) {
  const resolvers = new Map();
  const rejectors = new Set();
  const ipc = new _nodeIpc.IPCModule();
  Object.assign(ipc.config, nodeIpcConfig, {
    id
  });
  await new Promise((resolve, reject) => {
    ipc.connectTo(id, getPath(ipc, id), resolve);
    ipc.of[id].on('error', reject);
  });
  ipc.of[id].on('response', message => {
    const resolver = resolvers.get(message.id);
    if (!resolver) return;
    resolvers.delete(message.id);
    resolver(message.body);
  });
  ipc.of[id].on('error', error => {
    for (const rejector of rejectors) {
      rejector(error);
    }
  });

  function request(requestBody) {
    const messageId = randomId();
    const message = {
      id: messageId,
      body: requestBody
    };
    const promise = new Promise((resolve, reject) => {
      rejectors.add(reject);
      resolvers.set(messageId, resolve);
    });
    ipc.of[id].emit('request', message);
    return promise;
  }

  return {
    request
  };
}
//# sourceMappingURL=ipc.js.map